//
//    ケーブルチェッカ
//
//    copyright (c) 2012 T.Nakao
//    www.wsnak.com
//
//    12/12/04-12/11 新規
//    12/12/19 ショート、断線で表示を変えるように変更
//

/*
  出力シフトレジスタ(HC595)
    D2 OUTCLK(OUT)
    D3 RCK(OUT)
    D4 OUTDAT(OUT)

  入力シフトレジスタ(HC165)
    D5 SH/LD(OUT)
    D6 INCLK(OUT)
    D7 INDAT(IN)

  スイッチ入力
    D8 SW1
    D9 SW2

  ステータスLED
    D10 LED1(緑)
    D11 LED2(赤)

*/

typedef unsigned int word;

#define ON HIGH
#define OFF LOW

// ライブラリのリンク
#include <Wire.h>           // I2C (wI2cLcd使用時は必須)
#include <wCTimer.h>        // インターバルタイマ wCtcTimer2A
#include <wSwitch.h>        // スイッチドライバ w4Switch
#include <wDisplay.h>       // I2C-LCDドライバ wI2cLcd


wI2cLcd lcd(LCD_8BITMODE);  // I2C-LCDのインスタンス
wCtcTimer2A tm;             // タイマのインスタンス
w4Switch sw;                // スイッチのインスタンス

// ビット値変換テーブル
word bitShiftPat[16] = {
  0x0001,       // 1ピン
  0x0002,       // 2
  0x0004,       // 3
  0x0008,       // 4
  0x0010,       // 5
  0x0020,       // 6
  0x0040,       // 7
  0x0080,       // 8
  0x0100,       // 9
  0x0200,       // 10
  0x0400,       // 11
  0x0800,       // 12
  0x1000,       // 13
  0x2000,       // 14
  0x4000,       // 15
  0x8000        // 16
};


// D-SUBクロス用検査ビット・パターン
//   2と3、7と8が入れ替わり
word bitShiftPat2[9] = {
  0x0001,       // 1ピン
  0x0004,       // 3
  0x0002,       // 2
  0x0008,       // 4
  0x0010,       // 5
  0x0020,       // 6
  0x0080,       // 8
  0x0040,       // 7
  0x0100        // 9
};


// 検査モード関係
#define MAX_MODE 6        // 検査モード数
byte modeIndex = 0;       // 0～5 (モード数-1まで)
byte testMode[MAX_MODE] = {
  0x00,   // フリー 16ピン
  0x02,   // ピンプラグ 2P (モノラル)
  0x03,   // ピンプラグ 3P (ステレオ)
  0x04,   // USB 4ピン
  0x09,   // D-SUB 9ピン ストレート
  0x89    // D-USB 9ピン クロス
};
/*
  testMode[]の設定値
    下位4ビットがピン数、MSB=1の時D-SUB 9Pクロス
*/

// LCD 表示タイトル (検査モード名) (testMode[]に対応)
char *modeTitle[MAX_MODE] = {
  "FREE(MAX16)",
  "PLUG-MONO",
  "PLUG-STEREO",
  "USB",
  "D-SUB9P(STR)",
  "D-SUB9P(RVS)",
};


// フリーモードのピン毎の状態保持エリア
byte pinSts[16];

  // 出力シフトレジスタ(HC595) 操作マクロ
#define OUTCLK_H  digitalWrite(2, HIGH)   // D2 OUTCLK(OUT)
#define RCLK_H    digitalWrite(3, HIGH)   // D3 RCK(OUT)
#define OUTDAT_1  digitalWrite(4, HIGH)   // D4 OUTDAT(OUT)
#define OUTCLK_L  digitalWrite(2, LOW)    // D2 OUTCLK(OUT)
#define RCLK_L    digitalWrite(3, LOW)    // D3 RCK(OUT)
#define OUTDAT_0  digitalWrite(4, LOW)    // D4 OUTDAT(OUT)

  // 入力シフトレジスタ(HC165) 操作マクロ
#define SH_LD_SH  digitalWrite(5, HIGH)   // D5 SH/LD(OUT)
#define INCLK_H   digitalWrite(6, HIGH)   // D6 INCLK(OUT)
#define SH_LD_LD  digitalWrite(5, LOW)    // D5 SH/LD(OUT)
#define INCLK_L   digitalWrite(6, LOW)    // D6 INCLK(OUT)

#define INDAT     digitalRead(7)          // D7 INDAT(IN)

// LED点消灯マクロ
#define LED_GREEN_ON  sw.Led1(ON)
#define LED_RED_ON    sw.Led2(ON)
#define LED_GREEN_OFF sw.Led1(OFF)
#define LED_RED_OFF   sw.Led2(OFF)

char StrBuf[16];    // 文字列バッファ


//
//  初期化
//
void setup()
{
  byte i;

  // ドライバのインスタンスの初期化
  Wire.begin();                   // I2C初期化
  lcd.begin(16, 2);               // LCD初期化
  tm.init(125);                   // タイマ初期化 タイマ値設定 8ms

  // スイッチ、LEDの初期化
  sw.initSwitch(8, 9);            // SW1:D8, SW2:D9 ポート設定
  sw.onKeyPress(keyPress);        // スイッチ入力ハンドラ(ユーザ関数)の登録
  sw.initLed(10, 11);             // LED1:D10, LED2:D11 ポート設定

  // 出力シフトレジスタ(HC595) ポート設定
  pinMode(2, OUTPUT);   // D2 OUTCLK(OUT)
  pinMode(3, OUTPUT);   // D3 RCK(OUT)
  pinMode(4, OUTPUT);   // D4 OUTDAT(OUT)

  // 入力シフトレジスタ(HC165) ポート設定
  pinMode(5, OUTPUT);   // D5 SH/LD(OUT)
  pinMode(6, OUTPUT);   // D6 INCLK(OUT)
  pinMode(7, INPUT);    // D7 INDAT(IN)

  // 出力ポートの状態を初期化
  OUTCLK_L;       // D2 OUTCLK(OUT)
  RCLK_L;         // D3 RCK(OUT)
  OUTDAT_0;       // D4 OUTDAT(OUT)
  SH_LD_LD;       // D5 SH/LD(OUT)
  INCLK_L;        // D6 INCLK(OUT)

  // I2C LCDのテスト表示
  lcd.print("CABLE CHECKER");
  lcd.setCursor(0, 1);
  lcd.print("12/12/19");

  delay(1000);

  // LEDの点灯テスト表示
  LED_GREEN_ON;
  LED_RED_ON;
  delay(500);
  LED_GREEN_OFF;
  delay(500);
  LED_RED_OFF;

  delay(500);
  lcd.clear();
  lcd.print(modeTitle[modeIndex]);
}


//
//  メインループ
//
void loop() {
  // タイマ周期処理 (8ms)
  if(tm.checkTimeup()) {
    sw.keyProc();                 // キーセンス処理
  }
}


//
// キー入力ハンドラ
//   ボタンが押されたときにコールされるユーザ関数
//
void keyPress(byte keyval) {

  switch(keyval) {
    case 1:       // SW1 モード切替
      modeIndex = (modeIndex + 1) % MAX_MODE;     // 次のモードへ切り換え
      // モードタイトルを表示
      lcd.clear();
      lcd.print(modeTitle[modeIndex]);
      LED_GREEN_OFF;
      LED_RED_OFF;
      break;

    case 2:       // SW2 スタートスイッチ
      pinTest();  // テスト処理
      break;
    }
}


//
// テスト処理メイン
//
void pinTest(void) {
  byte mode;
  word result;

  lcd.setCursor(0, 1);
  lcd.print("                ");

  LED_RED_OFF;      // 赤
  LED_GREEN_OFF;    // 緑


  mode = testMode[modeIndex];
  if(mode == 0) {
    // 16ビット フリー・モード(合否判定なし)
    testReadEach();
  } else {
    // それ以外のテスト・モード
    if(mode & 0x80) {
      // D-SUB 9Pクロス
      result = checkPinsD9R();
    } else {
      // それ以外
      result = checkPins(mode);       // modoは検査ピン数を示す
    }

    if(result != 0) {
      // エラーあり
      LED_RED_ON;     // 赤 NG
    } else {
      LED_GREEN_ON;   // 緑 OK
    }
  }
}


//
// ピン・セット
//   検査出力の指定のピンを1本だけアクティブ(L)にする
//   bnum=0,.15
//
void portBitSet(byte bnum) {
  byte i;
  word outdat;

  outdat = bitShiftPat[bnum];

  OUTDAT_0;
  RCLK_L;
  delay(1);

  for(i = 0; i < 16; i++) {
    if(outdat & 0x8000) {     // MSB取り出し
      // "1"
      OUTDAT_1;
    } else {
      // "0"
      OUTDAT_0;
    }
    outdat <<= 1;         // 次のビット
    delay(1);             // セットアップタイム

    OUTCLK_H;             // シフトクロックパルス
    delay(1);
    OUTCLK_L;
    delay(1);
  }

  RCLK_H;                 // 出力ラッチパルス
  delay(1);
  RCLK_L;
}


//
// 検査入力ポートからピン状態の16ビット読み出し
//  結果 正論理(アクティブなビットが1)
//
word readPort(void) {
  byte i;
  word rslt = 0;
  int sts;

  SH_LD_LD;           // パラレルデータ ロード
  delay(10);
  SH_LD_SH;           // パラレルデータ ホールド
  delay(1);

  sts = INDAT;        // b15
  if(sts == HIGH) {
    rslt |= 1;        // LSBにb15の値をセット
  }

  // b14～b0
  for(i = 0; i < 15; i++) {
    rslt <<= 1;       // LSBを次の結果保存用に空ける
    delay(1);
    INCLK_H;          // シフトクロックパルス出力
    delay(1);
    INCLK_L;

    sts = INDAT;
    if(sts == HIGH) {   // b15,b14,,,b0の順に入力
      rslt |= 1;        // LSB set
    }
  }

  return rslt;
}


//
//  異常ピンのサーチ(1対1検査)
//    指定のピン数回繰り返し
//
//    pinnum:検査ピン数
//    戻り値:16ビット値 "0"のビットが正常、"1"のビットが異常
//
word checkPins(byte pinnum) {
  byte i;
  word result;
  word rslt_bmp = 0;    // 結果ビットマップ 異常ピンの対応ビットが"1"

  // 結果表示
  lcd.setCursor(0, 1);

  for(i = 0; i < pinnum; i++) {
    portBitSet(i);                  // 1ピンだけアクティブにする
    result = readPort();            // 16ピン分読み込む
    if(result == bitShiftPat[i]) {  // ビットパターンと比較
      // 一致 OK
      lcd.print('o');
    } else {
      // 不一致
      rslt_bmp |= bitShiftPat[i];   // 異常ビット保存

      // 断線か短絡かで表示を変える 121219
      if(result == 0) {
        // 断線(アクティブビットなし)
        lcd.print('B');
      } else {
        // ショート
        lcd.print('S');
      }
    }
  }

  return rslt_bmp;
}


//
// 異常ピンのサーチ(D-SUB 9Pクロス専用)
//
word checkPinsD9R(void) {
  byte i;
  word result;
  word rslt_bmp = 0;    // 結果ビットマップ 異常ピンの対応ビットが"1"

  // 結果表示
  lcd.setCursor(0, 1);

  for(i = 0; i < 9; i++) {
    portBitSet(i);                    // 1ピンだけアクティブにする
    result = readPort();              // 16ピン分読み込む
    if(result == bitShiftPat2[i]) {   // クロス用パターン
      // 一致 OK
      lcd.print('O');
    } else {
      // 不一致
      rslt_bmp |= bitShiftPat[i];     // 異常ビット保存

      // 断線か短絡かで表示を変える 121219
      if(result == 0) {
        // 断線(アクティブビットなし)
        lcd.print('B');
      } else {
        // ショート
        lcd.print('S');
      }
    }
  }

  return rslt_bmp;
}


//
// ピン並びの読み込み(16bit)
//
void testReadEach(void) {
  byte i, j;
  word rslt;
  byte pnum;

  lcd.setCursor(0, 0);
  lcd.print("0123456789ABCDEF");

  lcd.setCursor(0, 1);        // 2行目

  for(i = 0; i < 16; i++) {
    pinSts[i] = 0xFF;
  }

  for(i = 0; i < 16; i++) {   //
    portBitSet(i);            // 1ピンだけアクティブにする
    rslt = readPort();        // 16ピン分読み込む

    for(j = 0; j < 16; j++) {
      if(rslt & 1) {          // LSB取り出し
        // 導通あり
        pinSts[j] = i;        // カレントピン番号を設定
      }
      rslt >>= 1;             // 1ビット右シフト(次に取り出すビットをLSBへ)
    }
  }


  for(i = 0; i < 16; i++) {
    if(pinSts[i] == 0xFF) {
      lcd.print('_');             // 導通なし
    } else {
      lcd.print(pinSts[i], 16);   // ピン番号"0"～"F"表示
    }
  }
}
